
package agtweet;

import java.io.File;
import java.io.PrintStream;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author shauns
 */
public class AGTweet {

    final static String LOGNAME = "agtweetlog";
    
    final static class Seperators
    {
        final static String FOLLOW = " follows ";
        final static String TWEET  = "> ";
    }
    
    final static class Strings
    {
        /* 
         * NOTE: In a full production environment the strings below would want to 
         *       be externalised into a properties file, but that's going a bit 
         *       far for now.
         */
    
        static final String USERFILE_ERR_FILENOTFOUND     = "User file not found";

        final static String USERFILE_ERR_USER_EMPTYLINE  = "Empty user string";
        final static String USERFILE_ERR_USER_MISSINGSEP = "Missing seperator '" + AGTweet.Seperators.FOLLOW + "' in user string";

        final static String USERFILE_INFO_LOADED_NOERROR = "User file loaded with no errors";
        final static String USEFILE_INFO_LOADED_ERROR    = "User file loaded with errors";
        
        final static String TWEETFILE_INFO_LOADED_NOERROR = "User file loaded with no errors";
        final static String TWEETFILE_INFO_LOADED_ERROR   = "User file loaded with errors";                
        
        final static String TWEETFILE_ERR_EMPTYLINE  = "Empty tweet string";
        final static String TWEETFILE_ERR_MISSINGSEP = "Mssing seperator '" + Seperators.TWEET + "' in tweet string";
    
        final static String TWEET_ERR_TOOLONG_SKIP = "Tweet too long, skipping";
        final static String TWEET_ERR_TOOLONG_TRIM = "Tweet too long, trimming";       
        
        final static String TWEETLIST_ERR_NOSUCHUSER = "No such user";
        
        final static String AGTWEET_USERFILE_LOADERROR  = "Error loading userfile: ";
        final static String AGTWEET_TWEETFILE_LOADERROR = "Error loading tweetfile: ";
        
        final static String AGTWEET_USERFILE_NOTFOUND = "Userfile not found: ";
        final static String AGTWEET_TWEETFILE_NOTFOUND = "Tweetfile not found:";
        
        final static String USER_TWEETERROR = "Error publishing tweet";
        
        final static String TWEET_FORMAT_STRING = "\t@%1$s: %2$s\n";
    }

    final static class Config
    {
        enum LargeTweetMode
        {
            LARGETWEET_SKIP_WITHERROR,
            LARGETWEET_SKIP_SILENT,
            LARGETWEET_TRIM_WITHERROR,
            LARGETWEET_TRIM_SILENT
        }
        
        static LargeTweetMode _largeTweetMode   = LargeTweetMode.LARGETWEET_TRIM_SILENT;
        static int            _maxTweetLength   = 140;
        
        /**
         * Select Tweet mode
         */
        enum PrintTweetMode
        {
            PRINTTWEET_SCANMETHOD, 
            PRINTTWEET_PUBSUBMETHOD
        }
        
        static PrintTweetMode _printTweetMode = PrintTweetMode.PRINTTWEET_PUBSUBMETHOD;
        
        static boolean _selfFollowing     = true;
        static boolean _discoverFollowers = true; 
        
    }
 
    UserList  _userList  = null;
    TweetList _tweetList = null;
    
    private boolean _hasLoaded = false;
    
    public boolean hasLoaded() { return _hasLoaded; }
    
    static String padRight( String inString, int padLength )
    {
        return String.format( "%1$-" + padLength + "s", inString );        
    }    
    
    public void Load( String fullUserFilePath, 
                      String fullTweetFilePath ) throws 
                                                 UserList.UserFileException, 
                                                 TweetList.TweetFileException
    {
        File userFile  = new File(fullUserFilePath );  
        File tweetFile = new File(fullTweetFilePath);
        
        boolean bothFilesFound = true;
        
        if( !userFile.exists()  ){ bothFilesFound = false; Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, Strings.AGTWEET_USERFILE_NOTFOUND  + fullUserFilePath  ); }
        if( !tweetFile.exists() ){ bothFilesFound = false; Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, Strings.AGTWEET_TWEETFILE_NOTFOUND + fullTweetFilePath ); }
                
        if(bothFilesFound)
        {
            _userList  = new UserList ( userFile  );
            _tweetList = new TweetList( _userList );

            _tweetList.LoadTweets(tweetFile);

            _hasLoaded = true;
        }
    }

    private void PrintTweetsByScanMethod(PrintStream ps)
    {
        for( String username : _userList )
        {            
            try {
                User u = _userList.getUser(username);

                ps.println(u.getName());

                for( Tweet tweet : _tweetList )
                {
                    String tweetUsername = tweet.getUsername();
                    if( u.isFollowing(tweetUsername)
                        ||
                        ( Config._selfFollowing && u.getName().equals(tweetUsername) ) )
                    {
                        ps.format( Strings.TWEET_FORMAT_STRING, tweetUsername, tweet.getContents() );
                    }
                }

            } catch (UserList.UserListException ule) {
                Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, ule.getMessage(), ule);
            }            
        }
    }
    
    private void PrintTweetsByPubSubMethod(PrintStream ps)
    {
        if( !_userList.areFollowersPopulated() ||
            !_tweetList.areTweetsPublished() ) 
        { throw new RuntimeException("Users must be populated and tweets published before running PubSub Method" ); }
        
        for( String username :  _userList )
        {
            try {
                User u = _userList.getUser(username);

                ps.println(u.getName());

                for( Tweet tweet : u.getTweetsToSee() )
                {
                    ps.format( Strings.TWEET_FORMAT_STRING, tweet.getUsername(), tweet.getContents() );
                }

            } catch (UserList.UserListException ule) {
                Logger.getLogger(AGTweet.LOGNAME).log(Level.SEVERE, ule.getMessage(), ule);
            }                        
        }
    }
    
    public void PrintTweets(PrintStream ps)
    {
        if( !_hasLoaded ){ throw new RuntimeException("Must load before printing!"); }
        
        switch(Config._printTweetMode)
        {
            case PRINTTWEET_SCANMETHOD  : { PrintTweetsByScanMethod  (ps); break; }
            case PRINTTWEET_PUBSUBMETHOD: { PrintTweetsByPubSubMethod(ps); break; }           
        }        
        
        ps.close();
    }
    
    void DoPublishing()
    {
        _userList.PopulateFollowers();
        _tweetList.PublishTweets();
    }
    
}
