/*
 * 
 */

package agtweet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.nio.file.FileSystems;
import java.time.Clock;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

/**
 *
 * @author shauns
 */
public class AGTweetRunner {
    
    private static void InitLog()
    {
        try {
            //ConsoleHandler ch = new ConsoleHandler();
            FileHandler    fh = new FileHandler("AGTweetLog.%u.%g.log");
                        
            //Logger.getLogger(AGTweet.LOGNAME).addHandler(ch);
            Logger.getLogger(AGTweet.LOGNAME).addHandler(fh);
                        
        } catch (Exception ex) {
            System.out.println("Cannot initialize log.");            
            System.exit(-1);
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws IOException {
        
        InitLog();

        String userFileName  = "user.txt";
        String tweetFileName = "tweet.txt";
        String outFileName   = "";
        String testFileName  = "";
        
        int tweetUserArgNum = 2;
        int outArgNum       = 3;
        int testArgNum      = 4;
        
        if( args[0].charAt(0) == '-' )
        {
            if(args[0].equals("-pubsub")){ AGTweet.Config._printTweetMode = AGTweet.Config.PrintTweetMode.PRINTTWEET_PUBSUBMETHOD; }
            if(args[0].equals("-scan"  )){ AGTweet.Config._printTweetMode = AGTweet.Config.PrintTweetMode.PRINTTWEET_SCANMETHOD  ; }
            
            tweetUserArgNum++; outArgNum++; testArgNum++;
        }

        if(args.length>=tweetUserArgNum)
        {
            userFileName  = args[tweetUserArgNum-2]; 
            tweetFileName = args[tweetUserArgNum-1];
        }
        
        if(args.length>=outArgNum)
        {
            outFileName = args[outArgNum-1];
        }
        
        if(args.length==testArgNum)
        {
            testFileName = args[testArgNum-1];
        }
        
        AGTweet agt = new AGTweet();
        
        String workingDir = System.getProperty("user.dir");
        
        String userFilePath  = workingDir + "\\" + userFileName;
        String tweetFilePath = workingDir + "\\" + tweetFileName;
        String outFilePath   = workingDir + "\\" + outFileName;
        String testFilePath  = workingDir + "\\" + testFileName;
        
        System.out.println(workingDir);
        
        Logger l = Logger.getLogger(AGTweet.LOGNAME);
        
        try {
            PrintStream pout = 
                    outFileName.equals("") ? 
                    System.out : 
                    new PrintStream(outFilePath);
                        
            l.log(Level.INFO, "Loading...");
            agt.Load( userFilePath, tweetFilePath );
            
            if(agt.hasLoaded())
            {
                if( AGTweet.Config._printTweetMode == AGTweet.Config.PrintTweetMode.PRINTTWEET_PUBSUBMETHOD )
                {
                    l.log(Level.INFO, "Publishing...");
                    agt.DoPublishing();
                }

                l.log(Level.INFO, "Printing...");
                agt.PrintTweets(pout);
            }

            if(!testFileName.equals(""))
            {
                l.log(Level.INFO, "Comparing...");
                CompareFiles(outFileName, testFileName);                
            }
        
            l.log(Level.INFO, "DONE");
            
        } catch (UserList.UserFileException ufe) {
            l.log(Level.SEVERE, ufe.getMessage(), ufe);
        } catch (TweetList.TweetFileException tfe) {
            l.log(Level.SEVERE, tfe.getMessage(), tfe);
        } catch (IOException ex) {
            l.log(Level.SEVERE, null, ex);
        }        
        
    }
    
    static void CompareFiles(String file1, String file2) throws IOException
    {
        BufferedReader buf1 = new BufferedReader(new FileReader(new File(file1)));
        BufferedReader buf2 = new BufferedReader(new FileReader(new File(file2)));
        
        String line1; int lineCnt1=0;
        String line2; int lineCnt2=0;
        
        boolean diff = false;
        
        while( buf1.ready() && buf2.ready() )
        {
            line1 = buf1.readLine(); lineCnt1++;
            line2 = buf2.readLine(); lineCnt2++;
            
            if(!line1.equals(line2))
            { 
                diff = true; 
                System.out.println("Difference on line: " + lineCnt1); 
                System.out.println(line1); 
                System.out.println(line2); 
            }
        }
                
        if(buf1.ready() || buf2.ready()){ diff = true; System.out.println("Different lengths: " + lineCnt1); }
        
        if(!diff){ System.out.println("SAME!"); }
    }
    
    
}
